@@ -33,6 +33,11 @@ class Parser(object):
     def __str__(self):
         return self.fmt
 
+    def keys(self):
+        """Get parameter names defined in the format string."""
+        convert_dict = get_convert_dict(self.fmt)
+        return convert_dict.keys()
+
     def parse(self, stri, full_match=True):
         '''Parse keys and corresponding values from *stri* using format
         described in *fmt* string.
@@ -67,7 +72,7 @@ class Parser(object):
     def validate(self, stri):
         """
         Validates that string *stri* is parsable and therefore complies with
-        this string format definition.  Useful for filtering strings, or to 
+        this string format definition.  Useful for filtering strings, or to
         check if a string if compatible before passing it to the
         parser function.
         """
@@ -83,9 +88,9 @@ class Parser(object):
         or loss in information.
 
         Note: This test only applies to sensible usage of the format string.
-        If string or numeric data is causes overflow, e.g. 
-        if composing "abcd" into {3s}, one to one correspondence will always 
-        be broken in such cases. This off course also applies to precision 
+        If string or numeric data is causes overflow, e.g.
+        if composing "abcd" into {3s}, one to one correspondence will always
+        be broken in such cases. This off course also applies to precision
         losses when using  datetime data.
         """
         return is_one2one(self.fmt)
@@ -142,6 +147,7 @@ formatter = StringFormatter()
 
 # taken from https://docs.python.org/3/library/re.html#simulating-scanf
 spec_regexes = {
+    'b': r'[-+]?[0-1]',
     'c': r'.',
     'd': r'[-+]?\d',
     'f': {
@@ -164,7 +170,7 @@ spec_regexes['E'] = spec_regexes['f']
 spec_regexes['g'] = spec_regexes['f']
 spec_regexes['X'] = spec_regexes['x']
 spec_regexes[''] = spec_regexes['s']
-allow_multiple = ['c', 'd', 'o', 's', '', 'x', 'X']
+allow_multiple = ['b', 'c', 'd', 'o', 's', '', 'x', 'X']
 fixed_point_types = ['f', 'e', 'E', 'g']
 # format_spec ::=  [[fill]align][sign][#][0][width][,][.precision][type]
 # https://docs.python.org/3.4/library/string.html#format-specification-mini-language
@@ -195,7 +201,7 @@ def _get_fixed_point_regex(regex_dict, width, precision):
 
 class RegexFormatter(string.Formatter):
     """String formatter that converts a format string to a regular expression.
-    
+
     >>> regex_formatter = RegexFormatter()
     >>> regex_str = regex_formatter.format('{field_one:5d}_{field_two}')
 
@@ -295,7 +301,7 @@ class RegexFormatter(string.Formatter):
         if fill is None:
             if width is not None and width[0] == '0':
                 fill = '0'
-            elif ftype in ['s', '', 'd']:
+            elif ftype in ['s', '', 'd', 'x', 'X', 'o', 'b']:
                 fill = ' '
 
         char_type = spec_regexes[ftype]
@@ -349,26 +355,30 @@ class RegexFormatter(string.Formatter):
         field_name, value = value
         return self.regex_field(field_name, value, format_spec)
 
-    def extract_values(self, fmt, stri, full_match=True):
-        """Extract information from string matching format.
 
-        Args:
-            fmt (str): Python format string to match against
-            stri (str): String to extract information from
-            full_match (bool): Force the match of the whole string. Default
-                to ``True``.
-
-        """
-        regex = self.format(fmt)
-        if full_match:
-            regex = '^' + regex + '$'
-        match = re.match(regex, stri)
-        if match is None:
-            raise ValueError("String does not match pattern.")
-        return match.groupdict()
+@lru_cache()
+def regex_format(fmt):
+    # We create a new instance of RegexFormatter here to prevent concurrent calls to
+    # format interfering with one another.
+    return RegexFormatter().format(fmt)
 
 
-regex_formatter = RegexFormatter()
+def extract_values(fmt, stri, full_match=True):
+    """Extract information from string matching format.
+
+    Args:
+        fmt (str): Python format string to match against
+        stri (str): String to extract information from
+        full_match (bool): Force the match of the whole string. Default
+            to ``True``.
+    """
+    regex = regex_format(fmt)
+    if full_match:
+        regex = '^' + regex + '$'
+    match = re.match(regex, stri)
+    if match is None:
+        raise ValueError("String does not match pattern.")
+    return match.groupdict()
 
 
 def _get_number_from_fmt(fmt):
@@ -394,6 +404,12 @@ def _convert(convdef, stri):
         result = _strip_padding(convdef, stri)
         if 'd' in convdef:
             result = int(result)
+        elif 'x' in convdef or 'X' in convdef:
+            result = int(result, 16)
+        elif 'o' in convdef:
+            result = int(result, 8)
+        elif 'b' in convdef:
+            result = int(result, 2)
         elif any(float_type_marker in convdef for float_type_marker in fixed_point_types):
             result = float(result)
 
@@ -447,7 +463,7 @@ def parse(fmt, stri, full_match=True):
 
     """
     convdef = get_convert_dict(fmt)
-    keyvals = regex_formatter.extract_values(fmt, stri, full_match=full_match)
+    keyvals = extract_values(fmt, stri, full_match=full_match)
     for key in convdef.keys():
         keyvals[key] = _convert(convdef[key], keyvals[key])
 
@@ -556,7 +572,7 @@ def globify(fmt, keyvals=None):
 def validate(fmt, stri):
     """
     Validates that string *stri* is parsable and therefore complies with
-    the format string, *fmt*.  Useful for filtering string, or to 
+    the format string, *fmt*.  Useful for filtering string, or to
     check if string if compatible before passing the string to the
     parser function.
     """
@@ -632,8 +648,8 @@ def is_one2one(fmt):
     or loss in information.
 
     Note: This test only applies to sensible usage of the format string.
-    If string or numeric data is causes overflow, e.g. 
-    if composing "abcd" into {3s}, one to one correspondence will always 
+    If string or numeric data is causes overflow, e.g.
+    if composing "abcd" into {3s}, one to one correspondence will always
     be broken in such cases. This of course also applies to precision
     losses when using  datetime data.
     """
@@ -663,7 +679,7 @@ def purge():
     is very limited.
 
     """
-    regex_formatter.format.cache_clear()
+    regex_format.cache_clear()
     get_convert_dict.cache_clear()
 
 
